//////////////////////////////////////////////////////////////////////
// SSTVGen.cpp: implementation of the CSSTVGen class.
//
//////////////////////////////////////////////////////////////////////
//SoundCard/Wavefile signal generator program
// Copyright 1999.    Moe Wheatley AE4JY  <ae4jy@mindspring.com>
//
//This program is free software; you can redistribute it and/or
//modify it under the terms of the GNU General Public License
//as published by the Free Software Foundation; either version 2
//of the License, or any later version.
//
//This program is distributed in the hope that it will be useful,
//but WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//GNU General Public License for more details.
//
//You should have received a copy of the GNU General Public License
//along with this program; if not, write to the Free Software
//Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
//
//////////////////////////////////////////////////////////////////////
//

#include "stdafx.h"
#include "WSCGen.h"
#include "SSTVGen.h"
#include <math.h>

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

//local defines

//#define SSTV_CONTINUOUS 1		//comment out for one time sending

#define SSTV_IDLE_STATE 0
#define SSTV_VIS_STATE 1
#define SSTV_HSYNC_STATE 2
#define SSTV_REDLINE_STATE 3
#define SSTV_GRNLINE_STATE 4
#define SSTV_BLULINE_STATE 5
#define SSTV_1100 253
#define SSTV_1200 254
#define SSTV_1300 255
#define SSTV_1500 0
#define SSTV_1900 126
#define SSTV_2300 252
#define SSTV_RED_PLANE 0
#define SSTV_GRN_PLANE 1
#define SSTV_BLU_PLANE 2
#define SSTV_PIXELS_PER_LINE 340
#define SSTV_S1_HSYNC_WIDTH .009	//HSYNC widths in seconds
#define SSTV_M1_HSYNC_WIDTH .005

//////// SSTV timing information from Dave Jones KB4YZ /////////
//  http://www.tima.com/~djones/line.txt
////////////////////////////////////////////////////////////////
// S1 speed is 140.1148942 Lines/Min with 9mSec Hsync
// S1 Line time = 60.0/140.1148942 - .009 = 0.41922
#define SSTV_S1_SEC_PER_PIXEL (0.41922/(3*SSTV_PIXELS_PER_LINE))
// M1 speed is 134.3947532 Lines/Min with 5mSec Hsync
// M1 Line time = 60.0/134.3947532 - .005 = 0.441446
#define SSTV_M1_SEC_PER_PIXEL (0.441446/(3*SSTV_PIXELS_PER_LINE))

#define SSTV_SLANT_ADJ 1.000000		//adjust + or - for correct slope

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CSSTVGen::CSSTVGen()
{
	m_2PI = 8.0 * atan(1.0);		//make a 2PI constant
	m_pImageArray = NULL;
}

CSSTVGen::~CSSTVGen()
{
	if(m_pImageArray)
	{
		delete []m_pImageArray;
		m_pImageArray = NULL;
	}

}

///////////+++++++++++++++++++++++++++++++++++++++++++++++++////////////////
///////////   S S T V  W A V E F O R M   G E N E R A T O R  ////////////////
///////////+++++++++++++++++++++++++++++++++++++++++++++++++////////////////

////////////////////////////////////////////////////////////////////////////
// Called to initialize the SSTV generator.
//  Fs is sample rate,  vis is the VIS code for Martin 1 or Scottie 1 modes
//  pDC is a pointer to the screen device context for use in test image drawing.
////////////////////////////////////////////////////////////////////////////
void CSSTVGen::InitSSTV( BYTE vis, INT Fs , CDC* pDC)
{
	m_SSTVMode = vis;
	if(m_SSTVMode == SSTV_S1_VIS_CODE )
		m_SSTVSecPerPixel = SSTV_S1_SEC_PER_PIXEL;
	else
		m_SSTVSecPerPixel = SSTV_M1_SEC_PER_PIXEL;

	m_SSTVPhaseIncConst= (m_2PI * ( (2300.0-1500.0)/252.0))/(double)Fs;
	m_SSTVPhaseIncOffset = (m_2PI * 1500.0)/(double)Fs;
	m_SSTVPhaseInc = m_SSTVPhaseIncOffset;
	m_SSTVSecPerSamp = SSTV_SLANT_ADJ /(double)Fs;	//number of Seconds per sample
	m_SSTVState = SSTV_IDLE_STATE;
	m_SSTVTime = 0.0;
	m_SSTVNextEventTime = 0.0;

	m_pImageArray = new BYTE[SSTV_PIXELS_PER_LINE ][SSTV_TOTAL_LINES][ SSTV_COLOR_PLANES ];
	memset( m_pImageArray, 0, SSTV_PIXELS_PER_LINE*SSTV_TOTAL_LINES*SSTV_COLOR_PLANES);
	CreateTestImage(pDC);
	m_Finished = FALSE;
}


////////////////////////////////////////////////////////////////////////////
// Calculates next n pData samples of SSTV signal.
//  returns TRUE if image transmission is finished.
////////////////////////////////////////////////////////////////////////////
BOOL CSSTVGen::CalcSSTV( double* pData , INT n)
{
static double t = 0.0;
INT i;
	for( i=0; i<n; i++ )
	{
		t += m_SSTVPhaseInc;
 		pData[i] = 1.414213562*sin( fmod(t,m_2PI) );
		m_SSTVTime += m_SSTVSecPerSamp;
		if( m_SSTVTime >= m_SSTVNextEventTime)	//time to service next
		{										//one-shot time?
			ProcSSTVState();
		}
	}
	return m_Finished;
}

////////////////////////////////////////////////////////////////////////////
// Called at various intervals to service SSTV state machine.
//
// m_SSTVNextEventTime specifies when this routine will be called again.
// m_SSTVState contains the present state of the image transmitter.
////////////////////////////////////////////////////////////////////////////
void CSSTVGen::ProcSSTVState( void )
{
	switch(m_SSTVState)
	{
		case SSTV_IDLE_STATE:			//here prior to starting image xmission
			m_VISState = 0;
			m_SSTVLineCount = 0;
			m_SSTVColorValue = SSTV_GRN_PLANE;
			m_SSTVPixelCount = 0;
			m_SSTVNextEventTime = m_SSTVTime + 2.0;		//2 seconds 1900 Hz pretone
			m_FrequencyCode = SSTV_1900;
			m_SSTVState = SSTV_VIS_STATE;
			break;
		case SSTV_VIS_STATE:			//here to transmit VIS code
			m_SSTVState = ProcVIS();
			break;
		case SSTV_HSYNC_STATE:			// here to xmit hsync pulse
			m_SSTVState = ProcHsync();
			break;
		case SSTV_REDLINE_STATE:		//here to xmit red image line
			m_SSTVState = ProcRedPixels();
			break;
		case SSTV_GRNLINE_STATE:		//here to xmit green image line
			m_SSTVState = ProcGrnPixels();
			break;
		case SSTV_BLULINE_STATE:		//here to xmit blue image line
			m_SSTVState = ProcBluPixels();
			break;
	}
	switch(m_FrequencyCode)		//convert BYTE frequency code to phase inc value
	{
		case SSTV_1300:				// 255 = 1300Hz
			m_SSTVPhaseInc = m_SSTVPhaseIncOffset*(1300.0/1500.0);
			break;
		case SSTV_1200:				// 254 = 1200Hz
			m_SSTVPhaseInc = m_SSTVPhaseIncOffset*(1200.0/1500.0);
			break;
		case SSTV_1100:				// 253 = 1100Hz
			m_SSTVPhaseInc = m_SSTVPhaseIncOffset*(1100.0/1500.0);
			break;
		default:				//0 to 252 maps to 1500 to 2300 Hz
			m_SSTVPhaseInc = m_SSTVPhaseIncConst * (double)m_FrequencyCode
										+ m_SSTVPhaseIncOffset;
			break;
	}
	
}

////////////////////////////////////////////////////////////////////////////
// shift out VIS code
////////////////////////////////////////////////////////////////////////////
INT CSSTVGen::ProcVIS( void )
{
INT ret;
	switch(m_VISState)
	{
		case 0:				//start bit 1200Hz
			m_VISCode = m_SSTVMode;
			m_VISState = 1;
			m_SSTVNextEventTime += .03;		//30 mSec
			m_FrequencyCode = SSTV_1200;
			ret = SSTV_VIS_STATE;
			break;
		case 9:				//stop bit 1200Hz
			m_VISState = 0;
			m_SSTVNextEventTime += .03;		//30 mSec
			m_FrequencyCode = SSTV_1200;
			m_SSTVLineCount = 0;
			m_SSTVColorValue = SSTV_GRN_PLANE;
			ret = SSTV_HSYNC_STATE;
			break;
		default:			// send out bits
			m_SSTVNextEventTime += .03;		//30 mSec
			ret = SSTV_VIS_STATE;
			if( m_VISCode & 0x1 )
				m_FrequencyCode = SSTV_1100;	// "1"
			else
				m_FrequencyCode = SSTV_1300;	// "0"
			m_VISCode = m_VISCode>>1;			//shift VIS code
			m_VISState++;
			break;
	}
	return ret;
}

////////////////////////////////////////////////////////////////////////////
// Horizontal Sync state
////////////////////////////////////////////////////////////////////////////
INT CSSTVGen::ProcHsync( void )
{
	INT ret;
	m_FrequencyCode = SSTV_1200;
	m_SSTVPixelCount = 0;
	if(m_SSTVMode == SSTV_S1_VIS_CODE )
	{
		m_SSTVNextEventTime += SSTV_S1_HSYNC_WIDTH;
		m_SSTVColorValue = SSTV_RED_PLANE;
		ret = SSTV_REDLINE_STATE;
	}
	else
	{
		m_SSTVNextEventTime += SSTV_M1_HSYNC_WIDTH;
		m_SSTVColorValue = SSTV_GRN_PLANE;
		ret = SSTV_GRNLINE_STATE;
	}
	return ret;
}

////////////////////////////////////////////////////////////////////////////
// Send out red Pixel state
////////////////////////////////////////////////////////////////////////////
INT CSSTVGen::ProcRedPixels( void )
{
INT ret;
	m_SSTVNextEventTime += m_SSTVSecPerPixel;
	m_FrequencyCode = m_pImageArray[m_SSTVPixelCount][m_SSTVLineCount][m_SSTVColorValue];
	if(	++m_SSTVPixelCount >= SSTV_PIXELS_PER_LINE)
	{
		m_SSTVPixelCount = 0;
		m_SSTVColorValue = SSTV_GRN_PLANE;
		if( m_SSTVLineCount++ < SSTV_TOTAL_LINES )
			if(m_SSTVMode == SSTV_S1_VIS_CODE )
				ret = SSTV_GRNLINE_STATE;
			else
				ret = SSTV_HSYNC_STATE;
		else
		{
#ifndef SSTV_CONTINUOUS
			m_Finished = TRUE;
#endif
			ret = SSTV_IDLE_STATE;
		}
	}
	else
	{
		ret = SSTV_REDLINE_STATE;
	}
	return ret;
}

////////////////////////////////////////////////////////////////////////////
// Send out green Pixel state
////////////////////////////////////////////////////////////////////////////
INT CSSTVGen::ProcGrnPixels( void )
{
INT ret;
	m_SSTVNextEventTime += m_SSTVSecPerPixel;
	m_FrequencyCode = m_pImageArray[m_SSTVPixelCount][m_SSTVLineCount][m_SSTVColorValue];
	if(	++m_SSTVPixelCount >= SSTV_PIXELS_PER_LINE)
	{
		m_SSTVPixelCount = 0;
		m_SSTVColorValue = SSTV_BLU_PLANE;
		ret = SSTV_BLULINE_STATE;
	}
	else
	{
		ret = SSTV_GRNLINE_STATE;
	}
	return ret;
}

////////////////////////////////////////////////////////////////////////////
// Send out blue Pixel state
////////////////////////////////////////////////////////////////////////////
INT CSSTVGen::ProcBluPixels( void )
{
INT ret;
	m_SSTVNextEventTime += m_SSTVSecPerPixel;
	m_FrequencyCode = m_pImageArray[m_SSTVPixelCount][m_SSTVLineCount][m_SSTVColorValue];
	if(	++m_SSTVPixelCount >= SSTV_PIXELS_PER_LINE)
	{
		m_SSTVPixelCount = 0;
		m_SSTVColorValue = SSTV_RED_PLANE;
		if(m_SSTVMode == SSTV_S1_VIS_CODE )
			ret = SSTV_HSYNC_STATE;
		else
			ret = SSTV_REDLINE_STATE;
	}
	else
	{
		ret = SSTV_BLULINE_STATE;
	}
	return ret;
}

/////////////////////  Test Image generation stuff  /////////////////////////
// try to use MFC CDC Class member functions to draw complex shapes and text
//  Unfortunately i can't figure out how to get a device context that isn't
// related to the screen or printer.  If the screen is only 16 colors then
// the drawing functions only do 16 colors.  Thank F&^%ing uSoft for the kludge.
//////////////////////////////////////////////////////////////////////////////

void CSSTVGen::CreateTestImage(CDC* pDC)
{
INT x,y,c;
CBitmap m_bmImage;
	m_bmImage.CreateCompatibleBitmap(pDC,SSTV_PIXELS_PER_LINE,SSTV_TOTAL_LINES );
CDC	MemDC;
	MemDC.CreateCompatibleDC( pDC);	//create memory DC pointer to the Image bitmap
CBitmap* pbmOld = MemDC.SelectObject( &m_bmImage );	//save off old
	MemDC.SetWindowOrg(0,0);
	MemDC.SetViewportExt( SSTV_PIXELS_PER_LINE, SSTV_TOTAL_LINES );
	MemDC.FillSolidRect( 0, 0, SSTV_PIXELS_PER_LINE, SSTV_TOTAL_LINES, RGB(0,0,0));
//////////////////  Draw a test pattern  ///////////////////////////////
#if 0	// the clean way except color space is for the screen not
		// for the 24 bit SSTV image array.
//  Horizontal Color wedges ( y=0 to y=140 )
INT i;
	for(i=0,c=0; i<256; i++,c+=0x10101)
		MemDC.FillSolidRect( 42+i, 0, 1, 16, c );			//White
	for(i=0; i<256; i++)
		MemDC.FillSolidRect( 42+(255-i), 20, 1, 20, i );	//Red
	for(i=0,c=0; i<256; i++,c+=0x100)
		MemDC.FillSolidRect( 42+i, 40, 1, 20, c );			//Green
	for(i=0,c=0; i<256; i++,c+=0x10000)
		MemDC.FillSolidRect( 42+(255-i), 60, 1, 20, c );	//Blue
	for(i=0,c=0; i<256; i++,c+=0x101)
		MemDC.FillSolidRect( 42+i, 80, 1, 20, c );			//Yellow
	for(i=0,c=0; i<256; i++,c+=0x10001)
		MemDC.FillSolidRect( 42+(255-i), 100, 1, 20, c );	//Magenta
	for(i=0,c=0; i<256; i++,c+=0x10100)
		MemDC.FillSolidRect( 42+i, 120, 1, 20, c );			//Cyan
#endif
//  Vertical Color Lines ( y=145 to y=160 )
	for(x=8; x<48; x+=8)
		{MemDC.MoveTo( x, 145 );DrawLine( x, 160, 1, RGB(255,255,255), &MemDC);}
	for(x=48; x<48*2; x+=8)
		{MemDC.MoveTo( x, 145 );DrawLine( x, 160, 4, RGB(255,0,0), &MemDC);}
	for(x=48*2; x<48*3; x+=8)
		{MemDC.MoveTo( x, 145 );DrawLine( x, 160, 4, RGB(0,255,0), &MemDC);}
	for(x=48*3; x<48*4; x+=8)
		{MemDC.MoveTo( x, 145 );DrawLine( x, 160, 4, RGB(0,0,255), &MemDC);}
	for(x=48*4; x<48*5; x+=8)
		{MemDC.MoveTo( x, 145 );DrawLine( x, 160, 4, RGB(255,255,0), &MemDC);}
	for(x=48*5; x<48*6; x+=8)
		{MemDC.MoveTo( x, 145 );DrawLine( x, 160, 4, RGB(255,0,255), &MemDC);}
	for(x=48*6; x<48*7; x+=8)
		{MemDC.MoveTo( x, 145 );DrawLine( x, 160, 4, RGB(0,255,255), &MemDC);}


//  Draw some text ( y=170 to y=250 )
RECT rect = {10,170,130,190};		//center text in this rectangle
	MemDC.SetBkColor( RGB(0, 0, 0) );
	MemDC.SetTextColor( RGB(0, 255, 0) );	//Green text
	MemDC.DrawText( "WSCGen", -1, &rect, DT_SINGLELINE|DT_VCENTER|DT_CENTER );
	rect.top=190; rect.bottom=210;//center text in this rectangle
	MemDC.SetTextColor( RGB(255, 0, 255) );	//Magenta text
	MemDC.DrawText( "Ver 1.7", -1, &rect, DT_SINGLELINE|DT_VCENTER|DT_CENTER );
	rect.top=210; rect.bottom=230;//center text in this rectangle
	MemDC.SetTextColor( RGB(0, 0, 255) );	//Blue text
	MemDC.DrawText( "by", -1, &rect, DT_SINGLELINE|DT_VCENTER|DT_CENTER );
	rect.top=230; rect.bottom=250;//center text in this rectangle
	MemDC.SetTextColor( RGB(150, 150, 150) );	//Grey text
	MemDC.DrawText( "AE4JY", -1, &rect, DT_SINGLELINE|DT_VCENTER|DT_CENTER );
//  Yellow Circle (  y=170 to y=250 )
CPen Pen;
	Pen.CreatePen( PS_SOLID,3, RGB(255,0,0) );
CPen* pOldPen = MemDC.SelectObject( &Pen );	//save off old
CBrush Brush;
	Brush.CreateHatchBrush( HS_DIAGCROSS, RGB(0,0,0) );
CBrush* pOldBrush = MemDC.SelectObject( &Brush );	//save off old
	MemDC.SetBkColor( RGB(255, 255, 0) );
	//center ellipse in this rectangle
	rect.left=130;rect.top=170;rect.right=210;rect.bottom=250;
	MemDC.Ellipse( &rect );
	//  Draw Polygons ( y=170 to y=250 )
CPen Pen2;
	Pen2.CreatePen( PS_SOLID,3, RGB(50,150,50) );
	MemDC.SelectObject( pOldPen );		//restore old
	pOldPen = MemDC.SelectObject( &Pen2 );	//save off old
	POINT TopPoints[11]={
		235,197,	275,170,	315,197,	315,224,	275,251,	275,224,
		235,197,	235,224,	275,251,	275,224,	315,197
	};
	MemDC.Polyline( TopPoints, 11 );
	MemDC.SelectObject( pOldPen );		//restore old
	MemDC.SelectObject( pOldBrush );	//restore old

////////////////////////////////////////////////////////////////////////
//copy pixel data from CBitmap object into the SSTV Image array.
	DrawColorWedges();
	for(y=145; y<SSTV_TOTAL_LINES; y++)
//	for(y=0; y<SSTV_TOTAL_LINES; y++)
	{
		for(x=0; x<SSTV_PIXELS_PER_LINE; x++)
		{
			c = MemDC.GetPixel( x, y);
			m_pImageArray[x][y][SSTV_RED_PLANE] = ((c&0x0000FF)*253)/256;
			c = c >>8;
			m_pImageArray[x][y][SSTV_GRN_PLANE] = ((c&0x0000FF)*253)/256;
			c = c >>8;
			m_pImageArray[x][y][SSTV_BLU_PLANE] = ((c&0x0000FF)*253)/256;
		}
	}
	MemDC.SelectObject( pbmOld );		//restore old
}

//////////////////////////////////////////////////////////////////////////////
// Draw a 'width' wide pixel 'color' line to x,y using pDC.
//////////////////////////////////////////////////////////////////////////////
void CSSTVGen::DrawLine( INT x, INT y,INT width, COLORREF color, CDC* pDC )
{
CPen Pen;
	Pen.CreatePen( PS_SOLID,width, color );
CPen* pOldPen = pDC->SelectObject( &Pen );	//save off old
	pDC->LineTo( x, y );
	pDC->SelectObject( pOldPen );		//restore old
}

//////////////////////////////////////////////////////////////////////////////
// Kludge to Directly draw into the Image Array instead of using CDC members
//////////////////////////////////////////////////////////////////////////////
void CSSTVGen::DrawColorWedges()
{
INT x,y;
COLORREF c;
BYTE tc;
	for( x=0,c=0; x<256; x++,c+=0x10101 )		//white wedge
		for( y=0; y<16; y++)
		{
			tc = (BYTE)( ( (c&0x0000FF)*253 )/256 );
			m_pImageArray[x+42][y][SSTV_RED_PLANE] = tc;
			tc = (BYTE)( ( ( (c>>8)&0x0000FF)*253 )/256 );
			m_pImageArray[x+42][y][SSTV_GRN_PLANE] = tc;
			tc = (BYTE)( ( ( (c>>16)&0x0000FF)*253 )/256 );
			m_pImageArray[x+42][y][SSTV_BLU_PLANE] = tc;
		}
	for( x=0,c=0; x<256; x++,c+=0x000001 )		//red wedge
		for( y=20; y<40; y++)
		{
			tc = (BYTE)( ( (c&0x0000FF)*253 )/256 );
			m_pImageArray[255-x+42][y][SSTV_RED_PLANE] = tc;
			tc = (BYTE)( ( ( (c>>8)&0x0000FF)*253 )/256 );
			m_pImageArray[255-x+42][y][SSTV_GRN_PLANE] = tc;
			tc = (BYTE)( ( ( (c>>16)&0x0000FF)*253 )/256 );
			m_pImageArray[255-x+42][y][SSTV_BLU_PLANE] = tc;
		}
	for(x=0,c=0; x<256; x++,c+=0x000100 )		//green wedge
		for( y=40; y<60; y++)
		{
			tc = (BYTE)( ( (c&0x0000FF)*253 )/256 );
			m_pImageArray[x+42][y][SSTV_RED_PLANE] = tc;
			tc = (BYTE)( ( ( (c>>8)&0x0000FF)*253 )/256 );
			m_pImageArray[x+42][y][SSTV_GRN_PLANE] = tc;
			tc = (BYTE)( ( ( (c>>16)&0x0000FF)*253 )/256 );
			m_pImageArray[x+42][y][SSTV_BLU_PLANE] = tc;
		}
	for( x=0,c=0; x<256; x++,c+=0x010000 )		//blue wedge
		for( y=60; y<80; y++)
		{
			tc = (BYTE)( ( (c&0x0000FF)*253 )/256 );
			m_pImageArray[255-x+42][y][SSTV_RED_PLANE] = tc;
			tc = (BYTE)( ( ( (c>>8)&0x0000FF)*253 )/256 );
			m_pImageArray[255-x+42][y][SSTV_GRN_PLANE] = tc;
			tc = (BYTE)( ( ( (c>>16)&0x0000FF)*253 )/256 );
			m_pImageArray[255-x+42][y][SSTV_BLU_PLANE] = tc;
		}
	for( x=0,c=0; x<256; x++,c+=0x010100 )		//cyan wedge
		for( y=80; y<100; y++)
		{
			tc = (BYTE)( ( (c&0x0000FF)*253 )/256 );
			m_pImageArray[x+42][y][SSTV_RED_PLANE] = tc;
			tc = (BYTE)( ( ( (c>>8)&0x0000FF)*253 )/256 );
			m_pImageArray[x+42][y][SSTV_GRN_PLANE] = tc;
			tc = (BYTE)( ( ( (c>>16)&0x0000FF)*253 )/256 );
			m_pImageArray[x+42][y][SSTV_BLU_PLANE] = tc;
		}
	for( x=0,c=0; x<256; x++,c+=0x010001 )		//magenta wedge
		for( y=100; y<120; y++)
		{
			tc = (BYTE)( ( (c&0x0000FF)*253 )/256 );
			m_pImageArray[255-x+42][y][SSTV_RED_PLANE] = tc;
			tc = (BYTE)( ( ( (c>>8)&0x0000FF)*253 )/256 );
			m_pImageArray[255-x+42][y][SSTV_GRN_PLANE] = tc;
			tc = (BYTE)( ( ( (c>>16)&0x0000FF)*253 )/256 );
			m_pImageArray[255-x+42][y][SSTV_BLU_PLANE] = tc;
		}
	for( x=0,c=0; x<256; x++,c+=0x000101 )		//yellow wedge
		for( y=120; y<140; y++)
		{
			tc = (BYTE)( ( (c&0x0000FF)*253 )/256 );
			m_pImageArray[x+42][y][SSTV_RED_PLANE] = tc;
			tc = (BYTE)( ( ( (c>>8)&0x0000FF)*253 )/256 );
			m_pImageArray[x+42][y][SSTV_GRN_PLANE] = tc;
			tc = (BYTE)( ( ( (c>>16)&0x0000FF)*253 )/256 );
			m_pImageArray[x+42][y][SSTV_BLU_PLANE] = tc;
		}

}
